@@ -33,6 +33,7 @@ CTF_TEST_DATA_FILE_NAME_DECL;
 int counter = 0;
 static int global_temp = 0;
 int attempts = 0;
+int captch = 0;
 
 int init_event_manager(int time)
 {
@@ -185,7 +186,7 @@ static char* get_captcha(guint32 captcha_sid)
  */
 int vk_req(xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
 {
-    int captch = 0;
+    captch = 0;
     int init_status = 0;
     gchar* remixmid = NULL;
     gchar* remixsid = NULL;
@@ -324,9 +325,10 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, char* remixmid, char* remixsid
     xmlChar* function = get_req_function(reqXml);
     int num = 0;
     int init_status = 0;
-
+    xmlXPathObject *Obj = NULL;
+    
     url = make_url(reqXml, remixmid, remixsid);
-
+    
     if (url == NULL || strlen(url) < 1) {
         if (g_strcmp0((gchar *)function, GET_BINARY_DATA) == 0 ) {
             counter = -1;
@@ -398,7 +400,15 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, char* remixmid, char* remixsid
                 }
                 if (g_strcmp0((gchar *)function, UPDATE_INBOX_MESSAGES) == 0 || 
                     g_strcmp0((gchar *)function, UPDATE_OUTBOX_MESSAGES) == 0) {
-                    num = parser_messages(tmp, doc);
+                    char* message_id = NULL;
+                    Obj = xpath("//Request/Params", reqXml);
+                    if(Obj != NULL) {
+                        xmlNodePtr node;
+                        node = Obj->nodesetval->nodeTab[0];
+                        message_id = xmlGetProp(node, "messageId");
+                        xmlXPathFreeObject(Obj);
+                    }
+                    num = parser_messages(tmp, doc, message_id);
                     g_free(tmp);
                     if (num == PAGE_SIZE && counter < MAX_COUNTER) {
                         g_debug("num_messages = %d", num);
@@ -418,9 +428,9 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, char* remixmid, char* remixsid
                         url = g_strdup_printf("http://userapi.com/data?act=inbox&from=0&to=%d&id=%s&sid=%s",num,remixmid,remixsid);
                         if(http_get(url, info_proxy, info_port, NULL, &tmp) == 200){
                             g_free(url);
-                            num = parser_messages(tmp, doc);
+                            num = parser_messages(tmp, doc, NULL);
                             g_free(tmp);
-			    read_message(doc, remixmid, remixsid);
+			    //read_message(doc, remixmid, remixsid);
                         } else {
                             info_message(DRV_NETW_ERROR_CODE, doc, DRV_NETW_ERROR, 1);
                         }
@@ -440,6 +450,12 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, char* remixmid, char* remixsid
                     g_free(function);
                     return 0;
                 }
+                if (g_strcmp0((gchar *)function, READ_MESSAGE) == 0) {
+                    info_message(DRV_MESS_READ_CODE, doc, DRV_MESS_READ, 0);
+                    g_free(tmp);
+                    g_free(function);
+                    return 0;
+                }
                 g_free(function);
                 break;
             case 5:
@@ -979,7 +995,7 @@ gchar* make_url(xmlDocPtr reqXml, char* remixmid, char* remixsid)
 {
     g_debug("make_url START");
     xmlChar* url;
-    const char* params[7];
+    const char* params[9];
     xmlDocPtr urlXml;
     xmlNodePtr root;
     //xmlDocDump(stdout, reqXml);
@@ -990,7 +1006,12 @@ gchar* make_url(xmlDocPtr reqXml, char* remixmid, char* remixsid)
     params[3] = g_strdup_printf("\"%s\"", remixsid);
     params[4] = g_strdup("COUNTER");
     params[5] = g_strdup_printf("%d", counter);
-    params[6] = NULL;
+    if(captch == 1) {
+        params[6] = g_strdup("CAPTCHA");
+        params[7] = g_strdup("1");
+        params[8] = NULL;
+    } else 
+        params[6] = NULL;
 
     xsltStylesheetPtr cur = NULL;
 
@@ -1019,7 +1040,11 @@ gchar* make_url(xmlDocPtr reqXml, char* remixmid, char* remixsid)
     g_free((gpointer*)params[3]);
     g_free((gpointer*)params[4]);
     g_free((gpointer*)params[5]);
-
+    if(captch == 1) {
+        g_free((gpointer*)params[6]);
+        g_free((gpointer*)params[7]);
+        captch = 0;
+    }
     g_printf("%s", url);
     g_debug("make_url END");
 
@@ -1133,7 +1158,7 @@ int captcha_message(xmlDocPtr doc, xmlDocPtr reqXml)
  * @param body - buffer for body of response
  * @return http server response code 
  */
-int http_get(const char* url, const char* proxy, long port, char** header, char** body)
+int http_get(const char* url, const char* proxy, long port, char** header, gchar** body)
 {
     CURL* c;
     CURLcode code;
@@ -1186,7 +1211,7 @@ int http_get(const char* url, const char* proxy, long port, char** header, char*
 
     if (body != NULL) {
         if (g_strcmp0(type2, "image/jpeg") == 0) {
-            *body = g_base64_encode(chunk.memory, chunk.size);
+            *body = g_base64_encode((guchar *)chunk.memory, chunk.size);
         } else {
     	    *body = g_strdup(chunk.memory);
         }
@@ -1313,7 +1338,6 @@ xmlXPathObject* xpath(char* req, xmlDocPtr doc)
 	g_debug("vk_xpath: end");
 
         xmlXPathFreeContext(Ctx);
-	//xmlFree(Ctx);
 
 	return Obj;
 
